@@ -12,10 +12,9 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c 1893 2007-07-09 22:23:28Z marvin $
  */
 /*
- *  Copyright (C) 2000-2002 Thomas Habets <thomas@habets.pp.se>
+ *  Copyright (C) 2000-2008 Thomas Habets <thomas@habets.pp.se>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -27,9 +26,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 //#include "config.h"
 #include <stdio.h>
@@ -61,6 +60,7 @@
 
 #if !defined(linux)
 #define HAVE_WEIRD_BSD 1
+#define FINDIF 1
 #endif
  
 #if defined(linux)
@@ -93,7 +93,7 @@
 #define IP_ALEN 4
 #endif
 
-const float version = 2.06f;
+const float version = 2.08f;
 
 static libnet_t *libnet = 0;
 
@@ -150,8 +150,10 @@ static void do_libnet_init(const char *ifname)
 /*
  *
  */
-static const char *arping_lookupdev_default(u_int32_t srcip, u_int32_t dstip,
-				     char *ebuf)
+static const char *
+arping_lookupdev_default(const char *ifname,
+			 u_int32_t srcip, u_int32_t dstip,
+			 char *ebuf)
 {
 #ifdef WIN32
 	WCHAR buf[LIBNET_ERRBUF_SIZE + PCAP_ERRBUF_SIZE];
@@ -170,7 +172,9 @@ static const char *arping_lookupdev_default(u_int32_t srcip, u_int32_t dstip,
 /*
  *
  */
-static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
+static const char *arping_lookupdev(const char *ifname,
+				    u_int32_t srcip,
+				    u_int32_t dstip,
 				    char *ebuf)
 {
 	FILE *f;
@@ -180,9 +184,9 @@ static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
 	char *p,*p2;
 	int n;
 
-	do_libnet_init(NULL);
-	libnet_addr2name4_r(dstip,0,buf2);
-	libnet_addr2name4_r(srcip,0,buf1);
+	do_libnet_init(ifname);
+	libnet_addr2name4_r(dstip,0,buf2,1024);
+	libnet_addr2name4_r(srcip,0,buf1,1024);
 
 	/*
 	 * Construct and run command
@@ -219,16 +223,70 @@ static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
 	*p2 = 0;
 	return p;
  failed:
-	return arping_lookupdev_default(srcip,dstip,ebuf);
+	return arping_lookupdev_default(ifname,srcip,dstip,ebuf);
+}
+#elif defined(FINDIF) && defined(HAVE_WEIRD_BSD)
+static
+const char *
+arping_lookupdev(const char *ifname,
+		 u_int32_t srcip, u_int32_t dstip, char *ebuf)
+{
+	FILE *f;
+	static char buf[10240];
+	char buf1[1024];
+	char *p,*p2;
+	int n;
+
+	do_libnet_init(ifname);
+	libnet_addr2name4_r(dstip,0,buf1, 1024);
+	//libnet_addr2name4_r(srcip,0,buf1);
+
+	/*
+	 * Construct and run command
+	 */
+	snprintf(buf, 1023, "/sbin/route -n get %s 2>&1",
+		 buf1);
+	if (!(f = popen(buf, "r"))) {
+		goto failed;
+	}
+	if (0 > (n = fread(buf, 1, sizeof(buf)-1, f))) {
+		pclose(f);
+		goto failed;
+	}
+	buf[n] = 0;
+	if (-1 == pclose(f)) {
+		perror("arping: pclose()");
+		goto failed;
+	}
+
+	/*
+	 * Parse out device
+	 */
+	p = strstr(buf, "interface: ");
+	if (!p) {
+		goto failed;
+	}
+
+	p+=11;
+
+	p2 = strchr(p, '\n');
+	if (!p2) {
+		goto failed;
+	}
+	*p2 = 0;
+	return p;
+ failed:
+	return arping_lookupdev_default(ifname,srcip,dstip,ebuf);
 }
 #else
 /*
  *
  */
-static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
+static const char *arping_lookupdev(const char *ifname,
+				    u_int32_t srcip, u_int32_t dstip,
 				    char *ebuf)
 {
-	return arping_lookupdev_default(srcip,dstip,ebuf);
+	return arping_lookupdev_default(ifname,srcip,dstip,ebuf);
 }
 #endif
 
@@ -339,7 +397,7 @@ static int is_mac_addr(const char *p)
 {
 	/* cisco-style */
 	if (3*5-1 == strlen(p)) {
-		int c;
+		unsigned int c;
 		for (c = 0; c < strlen(p); c++) {
 			if ((c % 5) == 4) {
 				if ('.' != p[c]) {
@@ -355,7 +413,7 @@ static int is_mac_addr(const char *p)
 	}
 	/* windows-style */
 	if (6*3-1 == strlen(p)) {
-		int c;
+		unsigned int c;
 		for (c = 0; c < strlen(p); c++) {
 			if ((c % 3) == 2) {
 				if ('-' != p[c]) {
@@ -989,7 +1047,7 @@ int main(int argc, char **argv)
 			break;
 		}
 		case 'S': // set source IP, may be null for don't-know
-			do_libnet_init(NULL);
+			do_libnet_init(ifname);
 			if (-1 == (srcip = libnet_name2addr4(libnet,
 							     optarg,
 							     LIBNET_RESOLVE))){
@@ -1026,7 +1084,7 @@ int main(int argc, char **argv)
 					"in MAC ping mode\n");
 				exit(1);
 			}
-			do_libnet_init(NULL);
+			do_libnet_init(ifname);
 			if (-1 == (dstip = libnet_name2addr4(libnet,
 							     optarg,
 							     LIBNET_RESOLVE))){
@@ -1061,7 +1119,7 @@ int main(int argc, char **argv)
 			mode = is_mac_addr(parm)?PINGMAC:PINGIP;
 		} else if (dstip_given) {
 			mode = PINGIP;
-			do_libnet_init(NULL);
+			do_libnet_init(ifname);
 			parm = strdup(libnet_addr2name4(dstip,0));
 			if (!parm) {
 				fprintf(stderr, "arping: out of mem\n");
@@ -1141,9 +1199,10 @@ int main(int argc, char **argv)
 	 */
 	if (!ifname) {
 		if (dont_use_arping_lookupdev) {
-			ifname = arping_lookupdev_default(srcip,dstip,ebuf);
+			ifname = arping_lookupdev_default(ifname,
+							  srcip,dstip,ebuf);
 		} else {
-			ifname = arping_lookupdev(srcip,dstip,ebuf);
+			ifname = arping_lookupdev(ifname,srcip,dstip,ebuf);
 		}
 		if (!ifname) {
 			fprintf(stderr, "arping: arping_lookupdev(): %s\n",
